% P53SNRpileup6.m SNR vs dead time
%
% COPYRIGHT:  Aprend Technology and Robert E. ALvarez, 2015. All rights reserved.
 
%% init vars
post_title = 'P53SNRpileup6';
% modify this for your computer
dirname = ['L:\Projects\Blog\Posts\' post_title '\'];
addpath([dirname 'code']);
% MakeBlogPostFileLists(post_title);

%% load data for plots
% save([dirname 'D2sVsEta1Feb15.mat'])
% load ExptD2sVsEta21May14-10AM.mat

%% Create the input spectrum and initialize parameters
tube_voltage = 80; % keV  
[~,~,specdat] = XrayTubeSpectrumTasmip(tube_voltage,'number_spectrum','clipzeros');
   % the mus for 'soft_tissue','bone_cortical'
materials = {'soft_tissue','bone_cortical'}; 
specdat.mus = MusFromMaterials(materials,specdat.egys);
   % counts per dead time parameters
netas = 25;
etas = geospace(1e-3,2,netas); % counts per dead time
   % transmited spectrum
   % use non-zero object thickness to get more realistic spectrum
thickness_object = 10;
a_object = [0.95; 0.05]; % 'soft_tissue','bone_cortical' for object
specdat.specnum = specdat.specnum.*exp(-specdat.mus*(thickness_object*a_object));
nbar_incident = 1000; % number of photons incident on detector
specdat.specnum = nbar_incident*specdat.specnum/trapz(specdat.egys,specdat.specnum);
   % PHA bins
nbins = [2 4];
nbins2test = numel(nbins);
   % optimal PHA bins set so equal number of counts per bin with no pileup
idx_thresholds = cell(nbins2test,2);
for k = 1:nbins2test
    idx_thresholds{k} = OptimNbinsThresholds(specdat,nbins(k)); 
end

%% get SNR vs etas at fixed thickness
   % set the following to true to compute optimal bins for each eta
get_optimal_bins = false;
if get_optimal_bins
    warning('computing optimal bins takes a LOT longer');
end
dA = [1; -1]; % for SNR from CRLB calculation   
  % pre-allocate results arrays
detector_names = {'ideal','N','Q','NQ'};
    % add PHA names
detector_names = [detector_names cellstr(num2str(nbins','PHA%d'))'];
   % pre-allocate for SNRs--include ideal
SNR_with_deadtime = zeros(numel(etas),length(detector_names)); 
   %  SNR with pileup computes SNR for ideal, N, Q, NQ detectors 
   % use it with eta = 0 for no pileup results
snrs = SNR_with_pileup(specdat, 0); 
SNR_no_deadtime = zeros(length(detector_names),1); % pre-allocate with zeros
SNR_no_deadtime(1:4) = [snrs.SNR_ideal, snrs.SNR_N, snrs.SNR_Q, snrs.SNR_NQ ];
   % pre-allocate for optimal PHA bins for each eta result
SNR_with_deadtime_opt = zeros(numel(etas),length(detector_names));
Ethreshold_opt = []; % optimal threshold energies
fprintf('starting calculations ...\n')
for keta =  1:netas
        % parameters for computation
      eta = etas(keta);
        %  SNR for ideal, N, Q, NQ detectors with pileup
      snrs = SNR_with_pileup(specdat, eta);
         % store result
      SNR_with_deadtime(keta,1:4) = [snrs.SNR_ideal, snrs.SNR_N, snrs.SNR_Q, snrs.SNR_NQ ];
         % PHA detectors
      for kbin = 1:nbins2test
          specdat.idx_threshold = idx_thresholds{kbin};      
          [~,pha_stats] = CRLB_PHA_with_pileup(specdat,eta);
          SNR_PHA = dA'*(pha_stats.F)*dA; % F is the inverse of the A-vector CRLB
             
          SNR_with_deadtime(keta, 4 + kbin) = SNR_PHA;
              % SNR_ with (approximately) zero deadtime for PHA detectors
          if keta == 1
              assert(abs(eta) < .002);
              SNR_no_deadtime(4 + kbin) =  SNR_PHA ;
          end

          if get_optimal_bins && (kbin==2)
                  %compute PHA4 SNR with optimal bins with pileup
              sp_w_pileup = SpectrumWithPileup(specdat,eta);
                   % recompute mus for energies in pileup spectrum
              sp_w_pileup.mus = MusFromMaterials(materials,sp_w_pileup.egys);
                 % use 2 steps to get optimal bins. 
                 % First coarse search
              [idx_threshold1,Ethreshold1] = OptimNbinsThresholdsSNR(sp_w_pileup,nbins(kbin), ...
                  'min_bin_width',2,'dynamic_range', 100,'verbose');     
                 % then fine search
              [idx_thresholdSNR,EthresholdSNR] = OptimNbinsThresholdsSNR(sp_w_pileup,nbins(kbin), ...
                  'min_bin_width',1,'dynamic_range', 100, ...
                  'close_egys',Ethreshold1,'close_dist',5,'verbose');
              specdat.idx_threshold =  idx_thresholdSNR;       
              [~,pha_stats] = CRLB_PHA_with_pileup(specdat,eta);
              SNR_PHA_opt = dA'*(pha_stats.F)*dA;
              SNR_with_deadtime_opt(keta, 4+kbin) = SNR_PHA_opt;
              Ethreshold_opt = [Ethreshold_opt EthresholdSNR];
          end
      end
      if get_optimal_bins
          fprintf('eta %g PHA4 SNR_ no opt %g with opt %g\n',eta, ...
              SNR_with_deadtime(keta, 4 + 2),  SNR_with_deadtime_opt(keta, 4 + 2));
      end
end
fprintf('\n')
    % compute ratios for plots
ndets = size(SNR_with_deadtime,2)-1;
  % SNR divided by SNR ideal 
ratios = SNR_with_deadtime(:,2:end)./repmat(SNR_with_deadtime(:,1),1,ndets);
ratios_no_deadtime = SNR_no_deadtime(2:end)./repmat(SNR_no_deadtime(1),ndets,1);
fprintf('finished\n');

%% plot ratios SNRwith-deadtime./SNR_-no-deadtime vs. eta
plot_colors = {'black','red','blue','lime','brown'}; 
plot_symbols = {'o','s','*','d','+'};
fontsiz = 14;
linewid = 1.5;
fh = figure;
   % set order to plot so legend order same as curves y-order
order2plot = [3 2 4 5 6] - 1; % index to ratios. 1st det type is 'ideal' so skip it
hold on
for k = 1:ndets
    kdet = order2plot(k);
   plot(etas,ratios(:,kdet)/ratios_no_deadtime(kdet),'linestyle','-', ...
       'marker',plot_symbols{kdet},'color', rgb(plot_colors{kdet}),'linewidth',linewid);
end
hold off
xlabel('\eta (photons per dead time)','fontsize',fontsiz);
ylabel('SNR^2/SNR^2_{0 dead time}','fontsize',fontsiz);
text(0.05,0.3,num2str(thickness_object,'SNR^2/SNR^2_{no deadtime}\nobject thickness %3.1f cm'),'fontsize',15);
hl = legend(detector_names(order2plot+1)','location','southeast');
axis([0 max(etas) 0 1.1])
set(gca,'fontsize',fontsiz,'linewidth',linewid);
set(fh,'color','w');

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'SNRdivSNR0';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% plot d^2/d^2_ideal vs. eta
fh = figure;
fontsiz = 16;
linewid = 1.5;
subplot(211)
  % plot PHA data
detectors2plot = [6 5 3] - 1; % index to ratios. 1st det type is 'ideal' so skip it
hold on
for kdet = detectors2plot
   plot(etas,ratios(:,kdet),'linestyle','-', ...
       'marker',plot_symbols{kdet},'color', rgb(plot_colors{kdet}),'linewidth',linewid);
end
hold off
xlabel('\eta (photons/dead time)','fontsize',fontsiz);
ylabel('SNR^2/SNR^2_{ideal}','fontsize',fontsiz);
text(0.2,0.85,num2str(thickness_object,'SNR^2/SNR^2_{ideal}\n80 kV spectrum\nobject thickness %3.1f cm'),'fontsize',fontsiz);
hl = legend(detector_names(detectors2plot+1)','location','northeast');
axis([0 max(etas) 0 1])
set(gca,'fontsize',fontsiz,'linewidth',linewid);

subplot(212)
  % plot N, Q, and NQ data
detectors2plot = [4 2 3] - 1; % index to ratios. 1st det type is 'ideal' so skip it
hold on
for kdet = detectors2plot
   plot(etas,ratios(:,kdet),'linestyle','-', ...
       'marker',plot_symbols{kdet},'color', rgb(plot_colors{kdet}),'linewidth',linewid);
end
hold off
xlabel('\eta (photons/dead time)','fontsize',fontsiz);
ylabel('SNR^2/SNR^2_{ideal}','fontsize',fontsiz);
hl = legend(detector_names(detectors2plot+1)','location','northeast');
axis([0 max(etas) 0 1])
set(gca,'fontsize',fontsiz,'linewidth',linewid);
psn = [200 200 550 1050];
set(fh,'color','w','position',psn);

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'SNRdivSNRideal';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% plot SNR with optimum PHA bins--need to run code with get_optimal_bins=true
SNR_with_deadtime_opt(:,1:3) = SNR_with_deadtime(:,1:3); % copy in the other dets
ratios_opt = SNR_with_deadtime_opt(:,2:end)./repmat(SNR_with_deadtime(:,1),1,ndets);
fontsiz = 16;
linewid = 1.5;
fh = figure;
psn = [200 200 500 1200];
set(fh,'color','w','position',psn);
hax = zeros(2,1);
hp1 = subplot( 211 );% first the SNR's
order2plot = [6 4 5 2 3] - 1; % index onto ratios. 1st det type is 'ideal' so skip it
hold on
for k = 1
    kdet = order2plot(k);
    plot(etas,ratios_opt(:,kdet),'linestyle','-', ...
       'marker','d','color', 'b','linewidth',linewid);
    plot(etas,ratios(:,kdet),'linestyle','-', ...
       'marker',plot_symbols{kdet},'color', rgb(plot_colors{kdet}),'linewidth',linewid);
end
   % plot Qs
kdet = 2;
plot(etas,ratios(:,kdet),'linestyle','-', ...
   'marker',plot_symbols{kdet},'color', rgb(plot_colors{kdet}),'linewidth',linewid);
hold off
axis([0 max(etas) 0 1])

hl = legend('optimal PHA bins','fixed PHA bins',detector_names{3},'location','best');
set(hl,'position',[0.5748 0.55 0.3194 0.1588]);
xlabel('\eta (photons per dead time)','fontsize',fontsiz);
ylabel('SNR^2/SNR^2_{ideal}','fontsize',fontsiz);
ht1 = text(0.2,0.85,'SNR with Fixed vs. optimal PHA bins','fontsize',fontsiz);
set(gca,'fontsize',fontsiz,'linewidth',linewid);
hax(1) = gca;

hp2 = subplot( 212 );% then the optimal bin energies
hold on
for kthresh = 3:-1:1
   plot(etas,Ethreshold_opt(kthresh,:),'linestyle','-', ...
      'marker',plot_symbols{kthresh},'color', rgb(plot_colors{kthresh}),'linewidth',linewid);
end
hold off
axis([0 2 0 100])
xlabel('\eta (photons per dead time)','fontsize',fontsiz);
ylabel('optimal inter-bin energy (keV)','fontsize',fontsiz);
ht2 = text(0.2,95,'Optimal inter-bin energies','fontsize',fontsiz);
set(gca,'fontsize',fontsiz,'linewidth',linewid);
hl2 = legend('bins 3-4','bins 2-3','bins 1-2' );
set(hl2,'position',[0.6176    0.31    0.2753    0.0667]);
hax(2) = gca;
set(hl,'position',[0.4606 0.75 0.4293 0.0667])
set(hp1,'position',[0.1560    0.5    0.7490    0.45])
set(hp2,'position',[0.1420    0.1100    0.7630 0.3]);

%% export the figure as png and eps
figno = sprintf('-f%d',fh);
fname = 'OptimumBins';
   % export as eps
exportfig(fh,[dirname fname '.eps'],'width',6,'LockAxes',0,'color','cmyk');
    % export as png
set(fh,'papertype','usletter')
print(figno,'-dpng','-cmyk','-r100',[dirname fname '.png']) % resolution 100 dpi

%% plot increase in variance with eta
fh = figure;
detypes = {'NQ','PHA'};
detnames = {'NQ','PHA 4'};
   % generate CRLB data with more eta values from formulas
netas4plot = 15;
etas4plot = linspace(0,0.8,netas4plot);
deadtimes4plot = etas4plot/rate;
kt4plot = 3; % index to nphotons to use for plot
nphotons = nphotons2do(kt4plot);
t_integrate = nphotons/rate;
spectran = sp1; % has idx_threshold and Ethreshold
spectran.specnum = nphotons*spectran.specnum_norm;
NQvars = zeros(netas4plot,2);
PHAvars = zeros(netas4plot,2);
for keta = 1:netas4plot
    [RA_NQ,nqstats] = CRboundNQNoLinearizeWithPileup(spectran,t_integrate,deadtimes4plot(keta));
    NQvars(keta,:) = diag(RA_NQ)';
    [RA_NK,pha_stats] = CRboundNoLinearizeWithPileup(spectran,t_integrate,deadtimes4plot(keta));
    PHAvars(keta,:) = diag(RA_NK)';
end
   % load Monte Carlo data from previous post
mc = load('L:\Projects\Blog\Posts\P52X1SNRpileup5_1\new_code.mat');
RA_NQ_all = mc.RA_NQ_all;
RA_PHA_all = mc.RA_PHA_all;
for ktype = 1:2
   subplot(2,1,ktype);
   mcdat = eval(['RA_' detypes{ktype} '_all']);
   crlbdat= eval(['CRLB_' detypes{ktype} '_all']);
   vardat = eval([detypes{ktype} 'vars']);
      % first plot the MC results
   hold on
   plot(etas,squeeze(mcdat(1,1,:,kt4plot)),'ok')
   plot(etas,squeeze(mcdat(2,2,:,kt4plot)),'*r')
   if ktype==1,legend('A_1','A_2','location','northwest');end
         % then the theoretical results
   plot(etas4plot,vardat(:,1),'-k','linewidth',1.5)
   plot(etas4plot,vardat(:,2),'-r','linewidth',1.5)
   hold off
   axis([0 .8 0 4]);
   if ktype==2, xlabel('\eta (photons per dead time)','fontsize',14);end
   text(0.5,3.5,detnames{ktype},'fontsize',14);
   ylabel('variance','fontsize',14);
   set(gca,'fontsize',12,'linewidth',1.2)
end
set(fh,'color','w','position',[250 100 450 850])
%% export the figure as eps
